'use strict';

const _ = require('lodash');
const Promise = require('bluebird');

const Service = require('../service');

class MockQueue extends Service {
  constructor(options, context) {
    super(options, context);

    this.subscriptions = [];
  }

  publish(key, message) {
    this.context.logger.info('queue message', {key: key, data: message})

    const subscribers = _.filter(this.subscriptions, subscription => {
      if (_.isString(key)) {
        return subscription.selector === key;
      }

      return _.every(subscription.selector, (v, k) => {
        return key[k] === v;
      });
    });

    return Promise.map(subscribers, subscriber => {
      return subscriber.handler({data: message, id: key});
    });
  }

  subscribe(selector, handler, options) {
    this.subscriptions.push({selector, handler});
    this.context.logger.info('subscribing to queue', {selector: selector});
    return () => {};
  }
}

module.exports = MockQueue;
